@@ -122,7 +122,7 @@
   </section>
 </chapter>
 
-   $Date: 2016-04-25 12:08:15 +0100 (Mon, 25 Apr 2016) $ $Revision: 39692 $
+   $Date$ $Revision$
    */ 
 
 #ifndef	INCLUDED_WEBSERVER_H
@@ -191,6 +191,20 @@
  *   <term>x-http-password</term>
  *   <desc>The password from the 'authorization' header if the request
  *     supplied HTTP basic authentication.</desc>
+ *   <term>x-cert-issuer</term>
+ *   <desc>The certificate issuer (RFC4514) if the request connection was
+ *     authenticated with a TLS/SSL certificate or if a secure proxy is in
+ *     use and the proxy set this header. NB the header from a secure proxy
+ *     takes precedence.</desc>
+ *   <term>x-cert-owner</term>
+ *   <desc>The certificate subject/owner (RFC4514) if the request connection
+ *     was authenticated with a TLS/SSL certificate or if a secure proxy is in
+ *     use and the proxy set this header. NB the header from a secure proxy
+ *     takes precedence.</desc>
+ *   <term>x-cert-owner-proxy</term>
+ *   <desc>Used if the proxy provides a certificate to identify itself.</desc>
+ *   <term>x-cert-issuer-proxy</term>
+ *   <desc>Used if the proxy provides a certificate to identify itself.</desc>
  *   <term>x-count-requests</term>
  *   <desc>The number of requests being processed at the point when
  *      this request started (includes this request).</desc>
@@ -313,8 +327,6 @@
  *   <desc>the address of the client host making the request</desc>
  *   <term>ident</term>
  *   <desc>not currently implemented ... '-' as placeholder</desc>
- *   <term>ident</term>
- *   <desc>not currently implemented ... '-' as placeholder</desc>
  *   <term>user</term>
  *   <desc>the remote user name from the authorization header, or '-'</desc>
  *   <term>timestamp</term>
@@ -364,6 +376,18 @@
  * class recognises the following defaults keys -
  * </p>
  * <deflist>
+ *   <term>WebServerFrameOptions</term>
+ *   <desc>A string defining the frame options setting for responses produced
+ *   by the server (application code can always override this).<br />
+ *   If this is not defined, the value <code>DENY</code> is used to prevent
+ *   responses from being presented inside frames.<br />
+ *   If this is defined as an empty string, no X-Frame-Options header is set
+ *   (unless application code explicitly sets the header in the response).<br />
+ *   Unless you use this option (or your application code explicitly
+ *   sets/removes the header), all responses will have the frame option DENY,
+ *   which will at least tend to keep security auditors who are afraid of
+ *   click-jacking attacks happy, even if it serves no other purpose.
+ *   </desc>
  *   <term>WebServerHosts</term>
  *   <desc>An array of host IP addresses to list the hosts permitted to
  *   send requests to the server.  If defined, requests from other hosts
@@ -440,6 +464,8 @@
   NSMutableDictionary   *_userInfoMap;
   NSLock                *_incrementalDataLock;
   NSMutableDictionary   *_incrementalDataMap;
+  NSUInteger            _strictTransportSecurity;
+  NSString              *_frameOptions;
   void			*_reserved;
 }
 
@@ -456,6 +482,13 @@
 /**
  * Same as the instance method of the same name.
  */
++ (NSUInteger) encodeURLEncodedForm: (NSDictionary*)dict
+                            charset: (NSString*)charset
+			       into: (NSMutableData*)data;
+
+/** DEPRECATED ... use +encodeURLEncodedForm:charset:into: instead.<br />
+ * Same as the instance method of the same name.
+ */
 + (NSUInteger) encodeURLEncodedForm: (NSDictionary*)dict
 			       into: (NSMutableData*)data;
 
@@ -614,9 +647,16 @@
  * Each value in the array may be a data object or a string.<br />
  * As a special case, a value may be a data object or a string rather
  * than an array ... this is treated like an array of one value.<br />
- * All non data keys and values are converted to data using
- * utf-8 encoding.<br />
+ * All non data keys and values are converted to data using the specified
+ * charset (or utf-8 if charset is nil/unrecognized or where the key/value
+ * cannot be represented using the specified charset).<br />
  * This method returns the number of values actually encoded.
+ */
+- (NSUInteger) encodeURLEncodedForm: (NSDictionary*)dict
+                            charset: (NSString*)charset
+			       into: (NSMutableData*)data;
+
+/** DEPRECATED ... use -encodeURLEncodedForm:charset:into: instead.<br />
  */         
 - (NSUInteger) encodeURLEncodedForm: (NSDictionary*)dict
 			       into: (NSMutableData*)data;
@@ -659,6 +699,12 @@
  */
 - (BOOL) isSecure;
 
+/**
+ * Returns YES if the server is running behiond a secure proxy inside
+ * a demilitarised zone (DMZ).
+ */
+- (BOOL) isTrusted;
+
 /**
  * Extracts request parameters from the HTTP query string and from the
  * request body (if it was application/x-www-form-urlencoded or
@@ -730,6 +776,7 @@
  * Loads a template file from disk and places it in aResponse as content
  * whose mime type is determined from the file extension using the
  * provided mapping (or a simple built-in default mapping if map is nil).<br />
+ * Text responses use utf-8 enmcoding.<br />
  * If you have a dedicated web server for handling static pages (eg images)
  * it is better to use that rather than vending static pages using this
  * method.  It's unlikely that this method can be as efficient as a dedicated
@@ -777,11 +824,18 @@
  * If anAddress is nil or empty, the receiver will listen on
  * all available network interfaces.<br />
  * If secure is nil then the receiver listens on aPort for HTTP requests.<br />
- * If secure is not nil, the receiver listens for HTTPS instead.<br />
- * If secure is a dictionary containing <code>CertificateFile</code>,
- * <code>KeyFile</code> and <code>Password</code> then the server will
- * use the specified certificate and key files (which it will access
- * using the password).<br />
+ * If secure is a dictionary, it must either contain <code>Proxy</code>
+ * with the value set to <code>YES</code> (to configure the receiver to
+ * accept HTTP connections but treat them as coming via a secure proxy),
+ * or it must contain <code>CertificateFile</code>,
+ * <code>KeyFile</code> and <code>Password</code> to configure the server
+ * to use the specified certificate and key files (which it will access
+ * using the password) to support HTTPS rather than HTTP. Alternatively
+ * both may be specified (if the connections are coming from a secure proxy
+ * and the connection between the proxy and the server is also secure.<br />
+ * If the dictionary also contains <code>HSTS</code> then this must be a
+ * non-negative integer value specifying the number of seconds to set
+ * in the Strict-Transport-Security header (defaults to 7 days).<br />
  * The <em>secure</em> dictionary may also contain other dictionaries
  * keyed on IP addresses, and if the address that an incoming connection
  * arrived on matches the key of a dictionary, that dictionary is used
@@ -953,11 +1007,23 @@
 - (void) setRoot: (NSString*)aPath;
 
 /** Configures a flag to say whether the receiver is running behind a
- * secure proxy and all connections are to be considered as having come
- * in via https.
+ * secure proxy (in a DMZ) and all connections are to be considered as
+ * having come in via https.<br />
+ * If this is not set, requests are not trusted and some headers may be
+ * deleted from them.
  */
 - (void) setSecureProxy: (BOOL)aFlag;
 
+/**
+ * Specifies the number of seconds HSTS is to be turned on for when responding
+ * to a request on a secure connection (including via a secure proxy).<br />
+ * The Strict-Transport-Security header is automatically set in the response
+ * to any incoming request (but code handling the request may alter that).<br />
+ * The default setting is 7 days (604800 seconds), while a setting of zero
+ * turns off HSTS.
+ */
+- (void) setStrictTransportSecurity: (NSUInteger)seconds;
+
 /**
  * Sets the maximum recursion depth allowed for substitutions into
  * templates.  This defaults to 4.
@@ -1023,6 +1089,13 @@
  */
 - (BOOL) streamData: (NSData*)data withResponse: (WebServerResponse*)response;
 
+/**
+ * Returns the number of seconds set for HSTS for this server.<br />
+ * This will be zero if the server is not using a secure connection or
+ * if HSTS has been disabled by the -setStrictTransportSecurity: method.
+ */
+- (NSUInteger) strictTransportSecurity;
+
 /**
  * Perform substitutions replacing the markup in aTemplate with the
  * values supplied by map and appending the results to the result.<br />
@@ -1061,6 +1134,11 @@
  * actually what it seems.
  */
 @interface      WebServerRequest : GSMimeDocument
+/** Convenience method returning the address of the client.<br />
+ * This is taken from the x-forwarded-for header if possible, but from the
+ * x-remote-address header if the request was not forwarded through a proxy.
+ */
+- (NSString*) address;
 @end
 
 /** Do not attempt to subclass the WebServerResponse class to add instance
